@@ -3,6 +3,8 @@
  * based on previous code from Rusty Russell <rusty@linuxcare.com.au>
  *
  * This coude is distributed under the terms of GNU GPL
+ *
+ * $Id: iptables-restore.c,v 1.10 2001/03/15 15:12:02 laforge Exp $
  */
 
 #include <getopt.h>
@@ -63,6 +65,21 @@ int parse_counters(char *string, struct ipt_counters *ctr)
 	return (sscanf(string, "[%llu:%llu]", &ctr->pcnt, &ctr->bcnt) == 2);
 }
 
+/* global new argv and argc */
+static char* newargv[1024];
+static int newargc;
+
+/* function adding one argument to newargv, updating newargc 
+ * returns true if argument added, false otherwise */
+static int add_argv(char *what) {
+	if (what && ((newargc + 1) < sizeof(newargv)/sizeof(char *))) {
+		newargv[newargc] = what;
+		newargc++;
+		return 1;
+	} else 
+		return 0;
+}
+
 int main(int argc, char *argv[])
 {
 	iptc_handle_t handle;
@@ -70,7 +87,6 @@ int main(int argc, char *argv[])
 	unsigned int line = 0;
 	int c;
 	char curtable[IPT_TABLE_MAXNAMELEN + 1];
-	char curchain[IPT_FUNCTION_MAXNAMELEN + 1];
 	FILE *in;
 
 	program_name = "iptables-restore";
@@ -107,17 +123,7 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 	else in = stdin;
-/*
-	handle = iptc_init("filter");
-	if (!handle)
-		exit_error(VERSION_PROBLEM,
-			   "can't initialize iptables-restore: %s",
-			   iptc_strerror(errno));
-
-	if (!clean_slate(&handle))
-		exit_error(OTHER_PROBLEM, "Deleting old chains: %s",
-			   iptc_strerror(errno));
-*/
+	
 	/* Grab standard input. */
 	while (fgets(buffer, sizeof(buffer), in)) {
 		int ret;
@@ -171,16 +177,14 @@ int main(int argc, char *argv[])
 					   program_name, line);
 				exit(1);
 			}
-			strncpy(curchain, chain, IPT_FUNCTION_MAXNAMELEN);
 
-			/* why the f... does iptc_builtin not work here ? */
 			/* FIXME: abort if chain creation fails --RR */
-//			if (!iptc_builtin(curchain, &handle)) {
-				DEBUGP("Creating new chain '%s'\n", curchain);
-				if (!iptc_create_chain(curchain, &handle))
-				DEBUGP("unable to create chain '%s':%s\n", curchain,
+			if (!iptc_builtin(chain, handle)) {
+				DEBUGP("Creating new chain '%s'\n", chain);
+				if (!iptc_create_chain(chain, &handle))
+				DEBUGP("unable to create chain '%s':%s\n", chain,
 					strerror(errno));
-//			}
+			}
 
 			policy = strtok(NULL, " \t\n");
 			DEBUGP("line %u, policy '%s'\n", line, policy);
@@ -220,54 +224,64 @@ int main(int argc, char *argv[])
 			ret = 1;
 
 		} else {
-			char *newargv[1024];
-			int i,a;
+			int a;
 			char *ptr = buffer;
-			char *ctrs = NULL;
-			struct ipt_counters count;
+			char *pcnt = NULL;
+			char *bcnt = NULL;
+			char *parsestart;
+
+			/* reset the newargv */
+			newargc = 0;
 
 			if (buffer[0] == '[') {
+				/* we have counters in our input */
 				ptr = strchr(buffer, ']');
 				if (!ptr)
 					exit_error(PARAMETER_PROBLEM,
 						   "Bad line %u: need ]\n",
 						   line);
-				ctrs = strtok(ptr, " \t\n");
+
+				pcnt = strtok(buffer+1, ":");
+				if (!pcnt)
+					exit_error(PARAMETER_PROBLEM,
+						   "Bad line %u: need :\n",
+						   line);
+
+				bcnt = strtok(NULL, "]");
+				if (!bcnt)
+					exit_error(PARAMETER_PROBLEM,
+						   "Bad line %u: need ]\n",
+						   line);
+
+				/* start command parsing after counter */
+				parsestart = ptr + 1;
+			} else {
+				/* start command parsing at start of line */
+				parsestart = buffer;
 			}
 
-			if (counters && ctrs) {
-
-				parse_counters(ctrs, &count);
+			add_argv(argv[0]);
+			add_argv("-t");
+			add_argv((char *) &curtable);
+			
+			if (counters && pcnt && bcnt) {
+				add_argv("--set-counters");
+				add_argv((char *) pcnt);
+				add_argv((char *) bcnt);
 			}
 
-			/* FIXME: Don't ignore counters. */
-
-			newargv[0] = argv[0];
-			newargv[1] = "-t";
-			newargv[2] = (char *) &curtable;
-			newargv[3] = "-A";
-			newargv[4] = (char *) &curchain;
-
-			/* strtok: a function only a coder could love */
-			for (i = 5; i < sizeof(newargv)/sizeof(char *); i++) {
-				if (!(newargv[i] = strtok(NULL, " \t\n")))
-					break;
-				ptr = NULL;
-			}
-			if (i == sizeof(newargv)/sizeof(char *)) {
-				fprintf(stderr,
-					"%s: line %u too many arguments\n",
-					program_name, line);
-				exit(1);
+			/* parse till end of line */
+			if (add_argv(strtok(parsestart, " \t\n"))) {
+				while (add_argv(strtok(NULL, " \t\n")));
 			}
 
 			DEBUGP("===>calling do_command(%u, argv, &%s, handle):\n",
-					i, curtable);
+					newargc, curtable);
 
-			for (a = 0; a <= i; a++)
+			for (a = 0; a <= newargc; a++)
 				DEBUGP("argv[%u]: %s\n", a, newargv[a]);
 
-			ret = do_command(i, newargv, &newargv[2], &handle);
+			ret = do_command(newargc, newargv, &newargv[2], &handle);
 		}
 		if (!ret) {
 			fprintf(stderr, "%s: line %u failed\n",
