@@ -95,9 +95,10 @@ static const char cmdflags[] = { 'I', 'D', 'D', 'R', 'A', 'L', 'F', 'Z',
 #define OPT_VIANAMEOUT	0x00100U
 #define OPT_FRAGMENT    0x00200U
 #define OPT_LINENUMBERS 0x00400U
-#define NUMBER_OF_OPT	11
+#define OPT_COUNTERS	0x00800U
+#define NUMBER_OF_OPT	12
 static const char optflags[NUMBER_OF_OPT]
-= { 'n', 's', 'd', 'p', 'j', 'v', 'x', 'i', 'o', 'f', '3'};
+= { 'n', 's', 'd', 'p', 'j', 'v', 'x', 'i', 'o', 'f', '3', 'c'};
 
 static struct option original_opts[] = {
 	{ "append", 1, 0, 'A' },
@@ -130,6 +131,7 @@ static struct option original_opts[] = {
 	{ "help", 2, 0, 'h' },
 	{ "line-numbers", 0, 0, '0' },
 	{ "modprobe", 1, 0, 'M' },
+	{ "set-counters", 1, 0, 'c' },
 	{ 0 }
 };
 
@@ -205,12 +207,15 @@ struct pprot {
 };
 
 /* Primitive headers... */
+/* defined in netinet/in.h */
+#if 0
 #ifndef IPPROTO_ESP
 #define IPPROTO_ESP 50
 #endif
 #ifndef IPPROTO_AH
 #define IPPROTO_AH 51
 #endif
+#endif
 
 static const struct pprot chain_protos[] = {
 	{ "tcp", IPPROTO_TCP },
@@ -387,6 +392,7 @@ exit_printhelp(void)
 "  --exact	-x		expand numbers (display exact values)\n"
 "[!] --fragment	-f		match second or further fragments only\n"
 "  --modprobe=<command>		try to insert modules using this command\n"
+"  --set-counters PKTS BYTES	set the counter during insert/append\n"
 "[!] --version	-V		print package version.\n");
 
 	/* Print out any special helps. A user might like to be able
@@ -974,8 +980,6 @@ register_match(struct iptables_match *me)
 
 	me->m = NULL;
 	me->mflags = 0;
-
-	opts = merge_options(opts, me->extra_opts, &me->option_offset);
 }
 
 void
@@ -1004,8 +1008,6 @@ register_target(struct iptables_target *me)
 	iptables_targets = me;
 	me->t = NULL;
 	me->tflags = 0;
-
-	opts = merge_options(opts, me->extra_opts, &me->option_offset);
 }
 
 static void
@@ -1314,8 +1316,12 @@ make_delete_mask(struct ipt_entry *fw)
 	unsigned char *mask, *mptr;
 
 	size = sizeof(struct ipt_entry);
-	for (m = iptables_matches; m && m->used; m = m->next)
+	for (m = iptables_matches; m; m = m->next) {
+		if (!m->used)
+			continue;
+
 		size += IPT_ALIGN(sizeof(struct ipt_entry_match)) + m->size;
+	}
 
 	mask = fw_calloc(1, size
 			 + IPT_ALIGN(sizeof(struct ipt_entry_target))
@@ -1324,7 +1330,10 @@ make_delete_mask(struct ipt_entry *fw)
 	memset(mask, 0xFF, sizeof(struct ipt_entry));
 	mptr = mask + sizeof(struct ipt_entry);
 
-	for (m = iptables_matches; m && m->used; m = m->next) {
+	for (m = iptables_matches; m; m = m->next) {
+		if (!m->used)
+			continue;
+
 		memset(mptr, 0xFF,
 		       IPT_ALIGN(sizeof(struct ipt_entry_match))
 		       + m->userspacesize);
@@ -1534,6 +1543,9 @@ static char *get_modprobe(void)
 		case -1: goto fail;
 		case 1024: goto fail; /* Partial read.  Wierd */
 		}
+		if (ret[strlen(ret)-1]=='\n') 
+			ret[strlen(ret)-1]=0;
+		close(procfile);
 		return ret;
 	}
  fail:
@@ -1585,8 +1597,12 @@ generate_entry(const struct ipt_entry *fw,
 	struct ipt_entry *e;
 
 	size = sizeof(struct ipt_entry);
-	for (m = matches; m && m->used; m = m->next)
+	for (m = matches; m; m = m->next) {
+		if (!m->used)
+			continue;
+
 		size += m->m->u.match_size;
+	}
 
 	e = fw_malloc(size + target->u.target_size);
 	*e = *fw;
@@ -1594,7 +1610,10 @@ generate_entry(const struct ipt_entry *fw,
 	e->next_offset = size + target->u.target_size;
 
 	size = 0;
-	for (m = matches; m && m->used; m = m->next) {
+	for (m = matches; m; m = m->next) {
+		if (!m->used)
+			continue;
+
 		memcpy(e->elems + size, m->m, m->m->u.match_size);
 		size += m->m->u.match_size;
 	}
@@ -1615,6 +1634,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 	const char *shostnetworkmask = NULL, *dhostnetworkmask = NULL;
 	const char *policy = NULL, *newname = NULL;
 	unsigned int rulenum = 0, options = 0, command = 0;
+	const char *pcnt = NULL, *bcnt = NULL;
 	int ret = 1;
 	struct iptables_match *m;
 	struct iptables_target *target = NULL;
@@ -1625,6 +1645,9 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 
 	memset(&fw, 0, sizeof(fw));
 
+	opts = original_opts;
+	global_option_offset = 0;
+
 	/* re-set optind to 0 in case do_command gets called
 	 * a second time */
 	optind = 0;
@@ -1646,7 +1669,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 	opterr = 0;
 
 	while ((c = getopt_long(argc, argv,
-	   "-A:C:D:R:I:L::F::Z::N:X::E:P:Vh::o:p:s:d:j:i:fbvnt:m:x",
+	   "-A:C:D:R:I:L::F::Z::N:X::E:P:Vh::o:p:s:d:j:i:fbvnt:m:xc:",
 					   opts, NULL)) != -1) {
 		switch (c) {
 			/*
@@ -1835,6 +1858,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 				target->t->u.target_size = size;
 				strcpy(target->t->u.user.name, jumpto);
 				target->init(target->t, &fw.nfcache);
+				opts = merge_options(opts, target->extra_opts, &target->option_offset);
 			}
 			break;
 
@@ -1889,6 +1913,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 			m->m->u.match_size = size;
 			strcpy(m->m->u.user.name, m->name);
 			m->init(m->m, &fw.nfcache);
+			opts = merge_options(opts, m->extra_opts, &m->option_offset);
 		}
 		break;
 
@@ -1926,6 +1951,32 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 			modprobe = optarg;
 			break;
 
+		case 'c':
+
+			set_option(&options, OPT_COUNTERS, &fw.ip.invflags,
+				   invert);
+			pcnt = optarg;
+			if (optind < argc && argv[optind][0] != '-'
+			    && argv[optind][0] != '!')
+				bcnt = argv[optind++];
+			else
+				exit_error(PARAMETER_PROBLEM,
+					"-%c requires packet and byte counter",
+					opt2char(OPT_COUNTERS));
+
+			if (sscanf(pcnt, "%llu", &fw.counters.pcnt) != 1)
+				exit_error(PARAMETER_PROBLEM,
+					"-%c packet counter not numeric",
+					opt2char(OPT_COUNTERS));
+
+			if (sscanf(bcnt, "%llu", &fw.counters.bcnt) != 1)
+				exit_error(PARAMETER_PROBLEM,
+					"-%c byte counter not numeric",
+					opt2char(OPT_COUNTERS));
+			
+			break;
+
+
 		case 1: /* non option */
 			if (optarg[0] == '!' && optarg[1] == '\0') {
 				if (invert)
@@ -1947,7 +1998,10 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 					       argv, invert,
 					       &target->tflags,
 					       &fw, &target->t))) {
-				for (m = iptables_matches; m && m->used; m = m->next) {
+				for (m = iptables_matches; m; m = m->next) {
+					if (!m->used)
+						continue;
+
 					if (m->parse(c - m->option_offset,
 						     argv, invert,
 						     &m->mflags,
@@ -1976,6 +2030,9 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 					strcpy(m->m->u.user.name, m->name);
 					m->init(m->m, &fw.nfcache);
 
+					opts = merge_options(opts,
+					    m->extra_opts, &m->option_offset);
+
 					optind--;
 					continue;
 				}
@@ -1988,8 +2045,13 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 		invert = FALSE;
 	}
 
-	for (m = iptables_matches; m && m->used; m = m->next)
+	for (m = iptables_matches; m; m = m->next) {
+		if (!m->used)
+			continue;
+
 		m->final_check(m->mflags);
+	}
+
 	if (target)
 		target->final_check(target->tflags);
 
